@@ -1,7 +1,7 @@
 #include "config.h"
 #define MOST_MAX_FILES 4096
 #define MOST_MAX_FILES 4096
-#define MOST_GUNZIP_TMP_FILE "/tmp/.zmost"
+#define MOST_GUNZIP_POPEN_FORMAT "gzip -dc %s"
 
 extern void most_reread_file (void);
 extern void most_read_to_line (int);
@@ -17,3 +17,4 @@ extern int most_head(char *, char *);
 extern char *Most_File_Ring[MOST_MAX_FILES];
 extern int Most_Num_Files;
 extern char Most_C_Dir[MAX_PATHLEN];
+extern int Most_Tail_Mode;
