@@ -21,6 +21,7 @@ extern void most_initialize_most(void);
 extern void most_exit_most (void);
 extern void most_exit_error (char *, ...);
 
+extern void most_usage (void);
 #define MOSTMALLOC most_malloc
 #define MOSTREALLOC most_realloc
 extern unsigned char *most_malloc(unsigned int);
@@ -28,8 +29,8 @@ extern unsigned char *most_realloc(unsigned char *, unsigned int);
 extern int Most_Tab_Width;
 
 #ifdef VMS
-#ifndef isdigit
-#define isdigit(x) \
+# ifndef isdigit
+#  define isdigit(x) \
       (((x >= '0') && (x <= '9')) ? 1 : 0)
-#endif
+# endif
 #endif
