@@ -1,3 +1,22 @@
+/*
+ This file is part of MOST.
+
+ Copyright (c) 1991, 1999 John E. Davis
+
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 #include "config.h"
 
 #include <stdio.h>
@@ -25,7 +44,6 @@
 #include "file.h"
 #include "line.h"
 
-
 char *Most_Global_Msg = "Press `Q' to quit, `H' for help, and SPACE to scroll.";
 
 Most_Window_Type *Most_Win;
@@ -66,7 +84,6 @@ void most_exit_minibuffer()
    Minibuffer_Selected = 0;
 }
 
-
 static void put_message_1 (char *what)
 {
    most_select_minibuffer ();
@@ -86,9 +103,9 @@ void most_flush_message (char *msg)
 void most_put_message ()
 {
    if (Beep_Mini) SLtt_beep();
-   
+
    put_message_1 ((char *) Most_Mini_Buf);
-   
+
    if (Beep_Mini)
      {
 	SLsmg_refresh ();
@@ -108,7 +125,6 @@ void most_put_message ()
 
 SLang_RLine_Info_Type *Most_RLI;
 
-
 static void rline_update (unsigned char *buf, int len, int col)
 {
    SLsmg_gotorc (SLtt_Screen_Rows - 1, 0);
@@ -122,14 +138,14 @@ static SLang_RLine_Info_Type  *init_readline (void)
 {
    unsigned char *buf = NULL;
    SLang_RLine_Info_Type *rli;
-   
+
    if ((NULL == (rli = (SLang_RLine_Info_Type *) SLMALLOC (sizeof(SLang_RLine_Info_Type))))
        || (NULL == (buf = (unsigned char *) SLMALLOC (256))))
      {
 	fprintf(stderr, "malloc error.\n");
 	exit(-1);
      }
-   
+
    SLMEMSET ((char *) rli, 0, sizeof (SLang_RLine_Info_Type));
    rli->buf = buf;
    rli->buf_len = 255;
@@ -138,38 +154,37 @@ static SLang_RLine_Info_Type  *init_readline (void)
    rli->getkey = SLang_getkey;
    rli->tt_goto_column = NULL;
    rli->update_hook = rline_update;
-   
+
    if (SLang_init_readline (rli) < 0)
      {
 	most_exit_error ("Unable to initialize readline library.");
      }
-   
+
    return rli;
 }
 
-
 int most_read_from_minibuffer(char *prompt, char *what)
 {
    int i;
-   
+
    if (Most_RLI == NULL)
      {
 	Most_RLI = init_readline ();
      }
-   
+
    Most_RLI->edit_width = SLtt_Screen_Cols - 1;
    Most_RLI->prompt = prompt;
    *Most_RLI->buf = 0;
    most_select_minibuffer();
-   
+
    /* do not use default.  The up arrow can always get it back. */
    if ((*what) && (what != Most_Search_Str))
      {
 	strcpy ((char *) Most_RLI->buf, what);
 	Most_RLI->point = strlen (what);
      }
-   
-   /* next routine returns -1 upon quit */ 
+
+   /* next routine returns -1 upon quit */
    i = SLang_read_line (Most_RLI);
    if ((i > 0) && !SLang_Error && !SLKeyBoard_Quit)
      {
@@ -178,15 +193,14 @@ int most_read_from_minibuffer(char *prompt, char *what)
      }
    if (SLKeyBoard_Quit) i = -1;
    SLang_Error = SLKeyBoard_Quit = 0;
-   
+
    most_exit_minibuffer();
-   
+
    if (i == -1) most_message ("Aborted!", 1);
-   
+
    return(i);
 }
 
-
 void most_clear_minibuffer()
 {
    Most_Mini_Buf[0] = '\0';
@@ -202,29 +216,28 @@ void most_check_minibuffer ()
        put_message_1 (Most_Global_Msg);
 }
 
-
 static int get_scroll(int *line)
 {
     /* line is the line we want at the top of the window if possible */
    int dtop, dbot,n,top,bot,wsize;
-   
+
    top = Most_Win->beg_line;
    wsize = Most_Win->bot - Most_Win->top; /* actually 1 less than window size */
    bot = top + wsize;
-   
+
    if ((*line == 1) && (top == 1))
      {
 	Most_Curs_Col = Most_Curs_Row = 1;
-	if (Most_Beg != Most_C_Pos)
+	if (Most_C_Offset != 0)
 	  {
-	     Most_C_Pos = Most_Beg;
+	     Most_C_Offset = 0;
 	     Most_C_Line = 1;
 	     return 0;
 	  }
 	most_message("Top of Buffer.",1);
 	return(0);
      }
-   
+
     /* handles endof file in a window */
    if ((bot > Most_Num_Lines) && *line > Most_C_Line)
      {
@@ -232,17 +245,17 @@ static int get_scroll(int *line)
 	most_message("End of Buffer.",1);
 	return(0);
      }
-   
+
    if (Most_Num_Lines <= wsize)     /* short file */
      {
 	*line = 1;
 	return(0);
      }
-   
+
    dtop = top - 1;
    dbot = Most_Num_Lines - bot;
    n = *line - top;
-   
+
    if ((n>0) && (dbot < n))
      {
 	n = dbot;
@@ -256,9 +269,9 @@ static int get_scroll(int *line)
 	if (!n)
 	  {
 	     Most_Curs_Col = Most_Curs_Row = 1;
-	     if (Most_Beg != Most_C_Pos)
+	     if (Most_C_Offset != 0)
 	       {
-		  Most_C_Pos = Most_Beg;
+		  Most_C_Offset = 0;
 		  Most_C_Line = 1;
 		  return 0;
 	       }
@@ -271,26 +284,29 @@ static int get_scroll(int *line)
 static void check_dirty_flag (void)
 {
    if (Most_Win->dirty_flag == 0) return;
-   
+
    Most_Win->dirty_flag = 0;
-   Most_Num_Lines = Most_Win->n_lines = most_count_lines (Most_Beg, Most_Eob);
+   /* 
+   Most_Num_Lines = most_count_lines (Most_Beg, Most_Eob);
+    */
+   Most_Win->n_lines = Most_Num_Lines;
 }
 
 void most_update_window (int line)
 {
    int save_line, save_col;
    unsigned long save_offset;
-   
+
    most_read_to_line (line);
    check_dirty_flag ();
-   
+
    if (Most_Column != Most_Win->col)
      {
 	if (Most_Column < 1) Most_Column = 1;
 	if (Most_Column != Most_Win->col)
 	  {
-	     save_offset = Most_Curs_Offset; 
-	     save_line = Most_Curs_Row; 
+	     save_offset = Most_Curs_Offset;
+	     save_line = Most_Curs_Row;
 	     save_col = Most_Curs_Col;
 	     most_redraw_window ();
 	     most_update_status ();
@@ -300,10 +316,9 @@ void most_update_window (int line)
 	  }
 	return;
      }
-   
-   
+
    (void) get_scroll (&line);
-   
+
    most_goto_line (line);
    most_redraw_window ();
    most_update_status ();
@@ -342,8 +357,6 @@ void most_update_windows (int line)
    if (flg) most_set_window(Most_Win);
 }
 
-
-
 void most_redraw_window (void)
 {
    int n, r;
@@ -351,35 +364,34 @@ void most_redraw_window (void)
    check_dirty_flag ();
 
    n = Most_Win->bot - Most_Win->top;
-   
+
    if ((Most_C_Line + n) > Most_Num_Lines)
      most_goto_line (Most_Num_Lines - n);
-   
-   Most_Curs_Offset = Most_C_Pos - Most_Beg;
+
+   Most_Curs_Offset = Most_C_Offset;
    Most_Win->curs_offset = Most_Win->top_offset = Most_Curs_Offset;
    Most_Win->beg_line = Most_C_Line;
    Most_Win->col = Most_Column;
    Most_Win->curs_col = Most_Curs_Col = 1;
    Most_Win->curs_line = Most_Curs_Row = 1;
-   
+
    r = Most_Win->top;
    most_goto_rc (r, 1);
-   
+
    most_display_line ();
-   
+
    while (n--)
      {
 	r++;
 	most_goto_rc (r, 1);
-	
+
 	if (most_forward_line(1))
 	  most_display_line();
 	else
 	  SLsmg_erase_eol ();
      }
-   
-   
-   Most_C_Pos = Most_Beg + Most_Win->top_offset;
+
+   Most_C_Offset = Most_Win->top_offset;
    Most_C_Line = Most_Win->beg_line;
 }
 
@@ -389,7 +401,6 @@ void most_redraw_window (void)
 #define MOST_S_OPT 8
 #define MOST_W_OPT 16
 
-
 /* associates current window with current buffer */
 void most_save_win_flags(Most_Window_Type *w)
 {
@@ -406,18 +417,17 @@ void most_save_win_flags(Most_Window_Type *w)
 void most_window_buffer()
 {
    Most_Win->beg_line = Most_C_Line;
-   Most_Win->top_offset = Most_C_Pos - Most_Beg;
+   Most_Win->top_offset = Most_C_Offset;
    Most_Win->col = Most_Column;
    Most_Win->buf = Most_Buf;
    Most_Selective_Display = 0;
    most_save_win_flags(Most_Win);
-   
+
    Most_Curs_Row = Most_Win->curs_line = 1;
    Most_Curs_Col = Most_Win->curs_col = 1;
    Most_Curs_Offset = Most_Win->curs_offset = Most_Win->top_offset;
 }
 
-
 static void restore_win_flags (void)
 {
    Most_V_Opt = Most_Win->flags & MOST_V_OPT;
@@ -429,15 +439,16 @@ static void restore_win_flags (void)
    Most_Selective_Display = Most_Win->display;
 }
 
-
 static Most_Window_Type *make_window(int r1,int r2)
 {
    Most_Window_Type *neew;
-   
+
    neew = (Most_Window_Type *) MOSTMALLOC(sizeof(Most_Window_Type));
    SLMEMSET ((char *) neew, 0, sizeof (Most_Window_Type));
+#if 0
    neew->status = (char *) MOSTMALLOC(256);
    SLMEMSET (neew->status, 0, SLtt_Screen_Cols + 1);
+#endif
    neew->col = Most_Column;
    neew->top = r1;
    neew->bot = r2;
@@ -453,111 +464,127 @@ int Most_Display_Inited = 0;
 void most_init_display ()
 {
    int h = SLtt_Screen_Rows;
-   
+
    if (Most_Display_Inited) return;
    Most_Display_Inited = 1;
-   
+
    most_get_term_dimensions(&SLtt_Screen_Cols, &SLtt_Screen_Rows);
-   
+
    if (Most_Win == NULL)
      {
 	Most_Top_Win = Most_Win = make_window(1,SLtt_Screen_Rows - 2);
 	Most_Win->prev = Most_Win->next = Most_Win;
      }
-   else if (h > SLtt_Screen_Rows) most_resize_display (0);
+   else if (h > SLtt_Screen_Rows) most_resize_display ();
    else Most_Top_Win->prev->bot = SLtt_Screen_Rows - 2;
-   
+
    SLsmg_init_smg ();
-   SLsmg_cls ();
    most_goto_rc (Most_Win->top,1);
    Most_Display_Inited = 1;
-   SLsmg_refresh ();
+   /* SLsmg_refresh (); */
 }
 
 void most_reset_display()
 {
    if (Most_Display_Inited == 0) return;
    SLsmg_reset_smg ();
-   
+
    if (Most_Restore_Width_To)
      {
 	most_set_width(Most_Restore_Width_To, 0);
 	Most_Restore_Width_To = 0;
      }
-   
+
    Most_Display_Inited = 0;
 }
 
 static void update_status1 (void)
 {
-   char str[30], ch, *strp;
-   static char neew[256];
-   int i,ii,r,x,line_p = 60, len;
-   int max_len;
-   Most_Buffer_Type *buf = Most_Win->buf;
-   
-   max_len = SLtt_Screen_Cols - 25;
-   r = Most_Win->bot + 1;
-   most_goto_rc(r,1);
-   
-   i = ii = 0;
-   neew[ii++] = '-';
-   if (Most_Win->lock) neew[ii++] = '*'; else neew[ii++] = '-';
-   strp = " MOST: ";
-   while(*strp != '\0') neew[ii++] = *strp++;
-   
-   len = strlen (buf->file);
-   if (len > max_len)
-     {
-        neew[ii++] = '.'; neew[ii++] = '.'; neew[ii++] = '.';
-	strcpy (&neew[ii], buf->file + (len - max_len));
-	ii += max_len;
-     }
+   char info[256];
+   char buf[32];
+   char *file;
+   unsigned int len;
+   unsigned int num_chars;
+   unsigned int field_width, info_len;
+   int r, x;
+
+   if (Most_Eob == Most_Beg) x = 100;
    else
      {
-	strcpy (&neew[ii], buf->file);
-	ii += len;
+	x = Most_C_Offset * 100;
+	x = x / (Most_Eob - Most_Beg);
      }
-   
-   while(ii < line_p) neew[ii++] = ' ';
-   x = (Most_C_Pos - Most_Beg) * 100;
-   if (buf->fd == -1)
-     {
-	if (Most_Eob == Most_Beg) x = 100; else x = x / (Most_Eob - Most_Beg);
-	if (Most_C_Line + (Most_Win->bot - Most_Win->top + 1) >= Most_Num_Lines) x = 100;
-     }
-   else
-     {
-	x = x / (buf->size);
-     }
-   
-   /* x = (Most_C_Line + Most_Win->bot - Most_Win->top) * 100; x = x / Most_Num_Lines; */
-   
-    /* for files with end of file above the bottom row (due to window manipulations) */
+
+   if (Most_C_Line + (Most_Win->bot - Most_Win->top + 1) >= Most_Num_Lines)
+     x = 100;
+
+   /* for files with end of file above the bottom row (due to window manipulations) */
    if (x > 100) x = 100;
+
+   sprintf (info, "(%d,%d) %d%%", Most_C_Line, Most_Column, x);
+
+   r = Most_Win->bot + 1;
+   most_goto_rc (r,1);
+
+   most_tt_reverse_video ();
+
+   buf[0] = '-';
+   if (Most_Win->lock) buf[1] = '*'; else buf[1] = '-';
+   strcpy (buf + 2, " MOST: ");
+
+   num_chars = strlen (buf);
+   SLsmg_write_nchars (buf, num_chars);
    
-   /* stdin may not be read in yet and I do not know how big it is. */
-   if (buf->fd == 0) sprintf(str,"(%d,%d) ",Most_C_Line,Most_Column);
+   /* So far num_chars have been written out.  We would like to put the
+    * line number information about info_len charcters from the RHS of the
+    * screen.  This leaves a field width of 
+    * SLtt_Screen_Cols - (16 + num_chars) for the filename.
+    */
+   info_len = strlen (info);
+   field_width = 12;
+   if (info_len > field_width)
+     field_width = info_len;
+   field_width = field_width + num_chars;
+   if (field_width > (unsigned int)SLtt_Screen_Cols)
+     field_width = 0;
    else
-     sprintf(str,"(%d,%d) %d%%",Most_C_Line,Most_Column,x);
-   
-   i = 0; while(ch = str[i++], ch != '\0') neew[ii++] = ch;
-   
-   while(ii < SLtt_Screen_Cols) neew[ii++] = '-';
-   neew[SLtt_Screen_Cols] = '\0';
-   
-   most_tt_reverse_video();
-   SLsmg_write_string (neew);
+     field_width = (unsigned int)SLtt_Screen_Cols - field_width;
+
+   file = Most_Win->buf->file;
+   len = strlen (file);
+
+   if (len >= field_width)
+     {
+	/* Filename does not fit.  So, only display the trailing characters
+	 * of the Filename.  Indicate this with ...
+	 */
+	if (field_width > 3)
+	  {
+	     buf[0] = '.';
+	     buf[1] = '.';
+	     buf[2] = '.';
+	     SLsmg_write_nchars (buf, 3);
+
+	     len += 3;
+	  }
+
+	file += (len - field_width);
+     }
+   SLsmg_write_string (file);
+   SLsmg_erase_eol ();
+
+   most_goto_rc (r, num_chars + field_width + 1);
+
+   SLsmg_write_nchars (info, info_len);
    most_tt_normal_video();
-   strcpy(Most_Win->status,neew);
 }
 
 void most_update_status (void)
 {
    check_dirty_flag ();
-   
+
    Most_C_Line = Most_Win->beg_line;
-   Most_C_Pos = Most_Beg + Most_Win->top_offset;
+   Most_C_Offset = Most_Win->top_offset;
    /* most_set_scroll_region(1,SLtt_Screen_Rows); */
    update_status1();
    /* most_set_scroll_region(Most_Win->top,Most_Win->bot); */
@@ -569,12 +596,12 @@ int most_split_window (void)
 {
    Most_Window_Type *new, *old;
    int b2,t2,b1, line;
-   
+
    b2 = Most_Win->bot;
    b1 = (Most_Win->bot + Most_Win->top) / 2 - 1;
    t2 = b1 + 2;
    if ((b1 == Most_Win->top) || (t2 == b2)) return(0);
-   
+
     /* line is top line of new window. */
    line = Most_Win->beg_line + t2 - Most_Win->top;
    old = Most_Win;
@@ -585,21 +612,22 @@ int most_split_window (void)
    new->next = Most_Win->next;
    new->prev = Most_Win;
    Most_Win->next = new;
-   
+
    new->beg_line = line;
    new->buf = Most_Buf;
+#if 0
     /* new window status line is at same position as old */
    strcpy(new->status,Most_Win->status);
+#endif
    return(1);
 }
 
-
 void most_two_windows()
 {
    int line;
    Most_Window_Type *nnew, *old;
    if (!most_split_window()) return;
-   
+
    old = Most_Win;
    nnew = Most_Win->next;
    line = nnew->beg_line;
@@ -607,12 +635,12 @@ void most_two_windows()
      {
 	most_other_window(1);
           /* since split window left nnew window undefined... */
-	Most_C_Pos = Most_Beg + old->top_offset;
+	Most_C_Offset = old->top_offset;
 	Most_C_Line = old->beg_line;
 	if (Most_Num_Lines <= nnew->bot - nnew->top + 1)
 	  {
 	     Most_C_Line = nnew->beg_line = 1;
-	     Most_C_Pos = Most_Beg;
+	     Most_C_Offset = 0;
 	     nnew->top_offset = 0;
 	     most_redraw_window();
 	     most_update_status();
@@ -620,7 +648,7 @@ void most_two_windows()
 	else if (line > Most_Num_Lines)
 	  {
 	     most_goto_line(Most_Num_Lines - nnew->bot + nnew->top);
-	     Most_Win->top_offset = Most_C_Pos - Most_Beg;
+	     Most_Win->top_offset = Most_C_Offset;
 	     Most_Win->beg_line = Most_C_Line;
 	     most_redraw_window();
 	     most_update_status();
@@ -628,13 +656,13 @@ void most_two_windows()
 	else
 	  {
 	     most_goto_line(line);
-	     Most_Win->top_offset = Most_C_Pos - Most_Beg;
+	     Most_Win->top_offset = Most_C_Offset;
 	     Most_Win->beg_line = Most_C_Line;
 	     most_update_window(Most_Num_Lines - nnew->bot + nnew->top);
 	  }
 	Most_Win->curs_line = 1;
 	Most_Win->curs_col = Most_Column;
-	Most_Win->curs_offset = Most_C_Pos - Most_Beg;
+	Most_Win->curs_offset = Most_C_Offset;
 	most_other_window(-1);
      }
    else
@@ -644,19 +672,18 @@ void most_two_windows()
 	nnew->beg_line = Most_C_Line;
 	nnew->curs_line = 1;
 	nnew->curs_col = Most_Column;
-	nnew->top_offset = Most_C_Pos - Most_Beg;
-	nnew->curs_offset = Most_C_Pos - Most_Beg;
+	nnew->top_offset = Most_C_Offset;
+	nnew->curs_offset = Most_C_Offset;
 	most_update_status();
 	Most_Win = old;
      }
    most_update_status();
 }
 
-
 static void delete_other_windows (void)
 {
    Most_Window_Type *w, *tmp;
-   
+
    if (Most_Win->next == Most_Win) return;
    w = Most_Win;
    Most_Win = Most_Win->next;
@@ -665,11 +692,15 @@ static void delete_other_windows (void)
      {
 	most_free_window_buffer(); /* needs a ring */
 	tmp = Most_Win->next;
+#if 0
 	/* if this is the bottom window, save its status line */
 	if (tmp == Most_Top_Win) strcpy(w->status,Most_Win->status);
+#endif
 	tmp->prev = Most_Win->prev;
 	Most_Win->prev->next = tmp;
+#if 0
 	SLFREE(Most_Win->status);
+#endif
 	SLFREE(Most_Win);
 	Most_Win = tmp;
      }
@@ -682,31 +713,32 @@ void most_one_window ()
     * sure that top, bot is set.
     */
    if (Most_Win->next != Most_Win)
-     {
-	delete_other_windows ();
-     }
+     delete_other_windows ();
+
    Most_Win->top = 1;
    Most_Win->bot = SLtt_Screen_Rows - 2;
+   if (Most_Win->bot < 1)
+     Most_Win->bot = 1;
+
    Most_Top_Win = Most_Win;
    most_redraw_window();
    most_update_status();
 }
 
-
 void most_set_window (Most_Window_Type *w)
 {
    Most_Win = w;
-   
+
    if (Most_Win->buf == NULL)
      {
 	Most_Win->buf = Most_Buf;
 	if (Most_Buf == NULL)
 	  return;
      }
-   
+
    most_switch_to_buffer(Most_Win->buf);
-   
-   Most_C_Pos = Most_Beg + Most_Win->top_offset;
+
+   Most_C_Offset = Most_Win->top_offset;
    Most_Curs_Offset = Most_Win->curs_offset;
    Most_Curs_Row = Most_Win->curs_line;
    Most_Curs_Col = Most_Win->curs_col;
@@ -720,13 +752,13 @@ void most_set_window (Most_Window_Type *w)
 void most_other_window(int n)
 {
    if (!n) return;
-   Most_Win->top_offset = Most_C_Pos - Most_Beg;
+   Most_Win->top_offset = Most_C_Offset;
    Most_Win->curs_offset = Most_Curs_Offset;
    Most_Win->curs_line = Most_Curs_Row;
    Most_Win->curs_col = Most_Curs_Col;
    Most_Win->beg_line = Most_C_Line;
    Most_Win->col = Most_Column;
-   
+
    most_save_win_flags(Most_Win);
    if (n < 0)
      while (n++) Most_Win = Most_Win->prev;
@@ -735,8 +767,6 @@ void most_other_window(int n)
    most_set_window(Most_Win);
 }
 
-
-
 /* kills window by moving lower window up */
 static void delete_as_top_window (void)
 {
@@ -747,7 +777,7 @@ static void delete_as_top_window (void)
    b2 = Most_Win->next->bot;
    Most_Win->prev->next = Most_Win->next;
    Most_Win->next->prev = Most_Win->prev;
-   
+
    most_other_window(1);
    Most_Win->top = t1;
    most_redraw_window();
@@ -759,9 +789,9 @@ void most_free_window_buffer (void)
 {
    Most_Window_Type *w;
    Most_Buffer_Type *b;
-   
+
    w = Most_Win->next;
-   
+
    while(Most_Win != w)
      {
 	if (Most_Win->buf == w->buf) return;
@@ -769,7 +799,14 @@ void most_free_window_buffer (void)
      }
    b = w->buf;
    if (b == NULL) return;
-   if ((b->fd != -1) && (b->fd != 0)) close(b->fd);
+   if ((b->fd != -1) /* && (b->fd != 0) */ )
+     {
+#if !defined(VMS)
+	if (b->fp != NULL) pclose (b->fp); 
+	else
+#endif
+	  close(b->fd);
+     }
    if (b->beg != NULL) SLFREE(b->beg);
    Most_Win->buf = NULL;
    SLFREE(b);
@@ -778,22 +815,24 @@ void most_free_window_buffer (void)
 void most_free_windows (void)
 {
    Most_Window_Type *w;
-   
+
    while (Most_Win != NULL)
      {
 	most_free_window_buffer ();
 	w = Most_Win->next;
 	w->prev = Most_Win->prev;
 	Most_Win->prev->next = w;
+#if 0
 	SLFREE (Most_Win->status);
+#endif
 	SLFREE (Most_Win);
 	if (w == Most_Win) w = NULL;
 	Most_Win = w;
      }
-   
+
    Most_Buf = NULL;
    Most_Beg = Most_Eob = NULL;
-   Most_C_Pos = NULL;
+   Most_C_Line = 0;
    Most_Num_Lines = Most_C_Line = 0;
 }
 
@@ -801,11 +840,11 @@ void most_delete_window (void)
 {
    int new_b, old_b;
    Most_Window_Type *w;
-   
+
    w = Most_Win;
    if (Most_Win->next == Most_Win) return;
    most_free_window_buffer();
-   
+
    if (Most_Win->next != Most_Top_Win)
      {
 	if (Most_Win == Most_Top_Win)
@@ -815,8 +854,9 @@ void most_delete_window (void)
 	  }
 	else
 	  delete_as_top_window();
-	
+#if 0
 	SLFREE(w->status);
+#endif
 	SLFREE(w);
 	return;
      }
@@ -826,13 +866,17 @@ void most_delete_window (void)
    most_other_window(-1);
    Most_Win->bot = new_b;
    most_redraw_window();
-   
+
+#if 0
    strcpy(Most_Win->status,w->status); /* share the same line */
-   
+#endif
+
    Most_Win->next = w->next;
    Most_Win->next->prev = Most_Win;
-   
+
+#if 0
    SLFREE (w->status);
+#endif
    SLFREE(w);
 
    most_update_status();
@@ -842,9 +886,9 @@ void most_redraw_display (void)
 {
    Most_Window_Type *w;
    int n,t;
-   
+
    SLsmg_cls ();
-   
+
    Last_Message = NULL;
    most_save_win_flags(Most_Win);
    w = Most_Win;
@@ -852,19 +896,19 @@ void most_redraw_display (void)
      {
 	Most_Win = Most_Win->next;
 	most_switch_to_buffer(Most_Win->buf);
-	Most_C_Pos = Most_Beg + Most_Win->top_offset;
+	Most_C_Offset = Most_Win->top_offset;
 	Most_C_Line = Most_Win->beg_line;
 	Most_Column = Most_Win->col;
-	
+
 	restore_win_flags();
 	n = Most_Win->bot - Most_Win->top;
-	
+
 	check_dirty_flag ();
-	
+
 	t = Most_Win->top;
 	most_goto_rc(t, 1);
 	most_display_line();
-	
+
 	while (n--)
 	  {
 	     t++;
@@ -872,13 +916,13 @@ void most_redraw_display (void)
 	     if (most_forward_line(1)) most_display_line();
 	  }
 	Most_C_Line = Most_Win->beg_line;
-	Most_C_Pos = Most_Beg + Most_Win->top_offset;
+	Most_C_Offset = Most_Win->top_offset;
 	update_status1();
      }
    while(Most_Win != w);
-   
+
    most_set_window(w);
-   
+
    if (Minibuffer_Selected)
      {
 	most_select_minibuffer ();
@@ -893,22 +937,20 @@ void most_toggle_lock()
    most_update_status();
 }
 
-
 void most_point_cursor (void)
 {
    int r, c;
-   
+
    c = Most_Curs_Col - Most_Column + 1;
    r = Most_Curs_Row;
-   
+
    if (r < 1) r = 1;
    else if (r > (Most_Win->bot - Most_Win->top + 1))
      r = Most_Win->bot - Most_Win->top + 1;
-   
+
    if (c > SLtt_Screen_Cols) c = SLtt_Screen_Cols;
    else if (c < 1) c = 1;
-   
+
    most_goto_rc (r + Most_Win->top - 1, c);
 }
 
-   
